@@ -1,7 +1,7 @@
 /*
- Copyright (c) 2004-2008 TrueCrypt Foundation. All rights reserved.
+ Copyright (c) 2004-2009 TrueCrypt Foundation. All rights reserved.
 
- Governed by the TrueCrypt License 2.6 the full text of which is contained
+ Governed by the TrueCrypt License 2.7 the full text of which is contained
  in the file License.txt included in TrueCrypt binary and source code
  distribution packages.
 */
@@ -12,12 +12,14 @@ extern "C" {
 
 BOOL ReadLocalMachineRegistryDword (char *subKey, char *name, DWORD *value);
 BOOL ReadLocalMachineRegistryMultiString (char *subKey, char *name, char *value, DWORD *size);
+BOOL ReadLocalMachineRegistryString (char *subKey, char *name, char *value, DWORD *size);
 int ReadRegistryInt (char *subKey, char *name, int defaultValue);
 char *ReadRegistryString (char *subKey, char *name, char *defaultValue, char *str, int maxLen);
 DWORD ReadRegistryBytes (char *path, char *name, char *value, int maxLen);
 void WriteRegistryInt (char *subKey, char *name, int value);
 BOOL WriteLocalMachineRegistryDword (char *subKey, char *name, DWORD value);
 BOOL WriteLocalMachineRegistryMultiString (char *subKey, char *name, char *multiString, DWORD size);
+BOOL WriteLocalMachineRegistryString (char *subKey, char *name, char *str, DWORD size);
 void WriteRegistryString (char *subKey, char *name, char *str);
 BOOL WriteRegistryBytes (char *path, char *name, char *str, DWORD size);
 void DeleteRegistryValue (char *subKey, char *name);
