@@ -1,7 +1,7 @@
 /*
- Copyright (c) 2008 TrueCrypt Foundation. All rights reserved.
+ Copyright (c) 2008-2009 TrueCrypt Foundation. All rights reserved.
 
- Governed by the TrueCrypt License 2.6 the full text of which is contained
+ Governed by the TrueCrypt License 2.7 the full text of which is contained
  in the file License.txt included in TrueCrypt binary and source code
  distribution packages.
 */
@@ -563,107 +563,118 @@ namespace TrueCrypt
 
 	Pkcs11Exception::operator string () const
 	{
-#define TC_CASE_STR(CODE) case CODE : return #CODE
-		switch (ErrorCode)
+		if (ErrorCode == CKR_OK)
+			return string();
+
+		static const struct
 		{
-		case CKR_OK: return string();
+			int ErrorCode;
+			const char *ErrorString;
+		} ErrorStrings[] =
+		{
+#			define TC_TOKEN_ERR(CODE) { CODE, #CODE },
 
-		TC_CASE_STR (CKR_CANCEL);
-		TC_CASE_STR (CKR_HOST_MEMORY);
-		TC_CASE_STR (CKR_SLOT_ID_INVALID);
-		TC_CASE_STR (CKR_GENERAL_ERROR);
-		TC_CASE_STR (CKR_FUNCTION_FAILED);
-		TC_CASE_STR (CKR_ARGUMENTS_BAD);
-		TC_CASE_STR (CKR_NO_EVENT);
-		TC_CASE_STR (CKR_NEED_TO_CREATE_THREADS);
-		TC_CASE_STR (CKR_CANT_LOCK);
-		TC_CASE_STR (CKR_ATTRIBUTE_READ_ONLY);
-		TC_CASE_STR (CKR_ATTRIBUTE_SENSITIVE);
-		TC_CASE_STR (CKR_ATTRIBUTE_TYPE_INVALID);
-		TC_CASE_STR (CKR_ATTRIBUTE_VALUE_INVALID);
-		TC_CASE_STR (CKR_DATA_INVALID);
-		TC_CASE_STR (CKR_DATA_LEN_RANGE);
-		TC_CASE_STR (CKR_DEVICE_ERROR);
-		TC_CASE_STR (CKR_DEVICE_MEMORY);
-		TC_CASE_STR (CKR_DEVICE_REMOVED);
-		TC_CASE_STR (CKR_ENCRYPTED_DATA_INVALID);
-		TC_CASE_STR (CKR_ENCRYPTED_DATA_LEN_RANGE);
-		TC_CASE_STR (CKR_FUNCTION_CANCELED);
-		TC_CASE_STR (CKR_FUNCTION_NOT_PARALLEL);
-		TC_CASE_STR (CKR_FUNCTION_NOT_SUPPORTED);
-		TC_CASE_STR (CKR_KEY_HANDLE_INVALID);
-		TC_CASE_STR (CKR_KEY_SIZE_RANGE);
-		TC_CASE_STR (CKR_KEY_TYPE_INCONSISTENT);
-		TC_CASE_STR (CKR_KEY_NOT_NEEDED);
-		TC_CASE_STR (CKR_KEY_CHANGED);
-		TC_CASE_STR (CKR_KEY_NEEDED);
-		TC_CASE_STR (CKR_KEY_INDIGESTIBLE);
-		TC_CASE_STR (CKR_KEY_FUNCTION_NOT_PERMITTED);
-		TC_CASE_STR (CKR_KEY_NOT_WRAPPABLE);
-		TC_CASE_STR (CKR_KEY_UNEXTRACTABLE);
-		TC_CASE_STR (CKR_MECHANISM_INVALID);
-		TC_CASE_STR (CKR_MECHANISM_PARAM_INVALID);
-		TC_CASE_STR (CKR_OBJECT_HANDLE_INVALID);
-		TC_CASE_STR (CKR_OPERATION_ACTIVE);
-		TC_CASE_STR (CKR_OPERATION_NOT_INITIALIZED);
-		TC_CASE_STR (CKR_PIN_INCORRECT);
-		TC_CASE_STR (CKR_PIN_INVALID);
-		TC_CASE_STR (CKR_PIN_LEN_RANGE);
-		TC_CASE_STR (CKR_PIN_EXPIRED);
-		TC_CASE_STR (CKR_PIN_LOCKED);
-		TC_CASE_STR (CKR_SESSION_CLOSED);
-		TC_CASE_STR (CKR_SESSION_COUNT);
-		TC_CASE_STR (CKR_SESSION_HANDLE_INVALID);
-		TC_CASE_STR (CKR_SESSION_PARALLEL_NOT_SUPPORTED);
-		TC_CASE_STR (CKR_SESSION_READ_ONLY);
-		TC_CASE_STR (CKR_SESSION_EXISTS);
-		TC_CASE_STR (CKR_SESSION_READ_ONLY_EXISTS);
-		TC_CASE_STR (CKR_SESSION_READ_WRITE_SO_EXISTS);
-		TC_CASE_STR (CKR_SIGNATURE_INVALID);
-		TC_CASE_STR (CKR_SIGNATURE_LEN_RANGE);
-		TC_CASE_STR (CKR_TEMPLATE_INCOMPLETE);
-		TC_CASE_STR (CKR_TEMPLATE_INCONSISTENT);
-		TC_CASE_STR (CKR_TOKEN_NOT_PRESENT);
-		TC_CASE_STR (CKR_TOKEN_NOT_RECOGNIZED);
-		TC_CASE_STR (CKR_TOKEN_WRITE_PROTECTED);
-		TC_CASE_STR (CKR_UNWRAPPING_KEY_HANDLE_INVALID);
-		TC_CASE_STR (CKR_UNWRAPPING_KEY_SIZE_RANGE);
-		TC_CASE_STR (CKR_UNWRAPPING_KEY_TYPE_INCONSISTENT);
-		TC_CASE_STR (CKR_USER_ALREADY_LOGGED_IN);
-		TC_CASE_STR (CKR_USER_NOT_LOGGED_IN);
-		TC_CASE_STR (CKR_USER_PIN_NOT_INITIALIZED);
-		TC_CASE_STR (CKR_USER_TYPE_INVALID);
-		TC_CASE_STR (CKR_USER_ANOTHER_ALREADY_LOGGED_IN);
-		TC_CASE_STR (CKR_USER_TOO_MANY_TYPES);
-		TC_CASE_STR (CKR_WRAPPED_KEY_INVALID);
-		TC_CASE_STR (CKR_WRAPPED_KEY_LEN_RANGE);
-		TC_CASE_STR (CKR_WRAPPING_KEY_HANDLE_INVALID);
-		TC_CASE_STR (CKR_WRAPPING_KEY_SIZE_RANGE);
-		TC_CASE_STR (CKR_WRAPPING_KEY_TYPE_INCONSISTENT);
-		TC_CASE_STR (CKR_RANDOM_SEED_NOT_SUPPORTED);
-		TC_CASE_STR (CKR_RANDOM_NO_RNG);
-		TC_CASE_STR (CKR_DOMAIN_PARAMS_INVALID);
-		TC_CASE_STR (CKR_BUFFER_TOO_SMALL);
-		TC_CASE_STR (CKR_SAVED_STATE_INVALID);
-		TC_CASE_STR (CKR_INFORMATION_SENSITIVE);
-		TC_CASE_STR (CKR_STATE_UNSAVEABLE);
-		TC_CASE_STR (CKR_CRYPTOKI_NOT_INITIALIZED);
-		TC_CASE_STR (CKR_CRYPTOKI_ALREADY_INITIALIZED);
-		TC_CASE_STR (CKR_MUTEX_BAD);
-		TC_CASE_STR (CKR_MUTEX_NOT_LOCKED);
-		TC_CASE_STR (CKR_NEW_PIN_MODE);
-		TC_CASE_STR (CKR_NEXT_OTP);
-		TC_CASE_STR (CKR_FUNCTION_REJECTED);
+			TC_TOKEN_ERR (CKR_CANCEL)
+			TC_TOKEN_ERR (CKR_HOST_MEMORY)
+			TC_TOKEN_ERR (CKR_SLOT_ID_INVALID)
+			TC_TOKEN_ERR (CKR_GENERAL_ERROR)
+			TC_TOKEN_ERR (CKR_FUNCTION_FAILED)
+			TC_TOKEN_ERR (CKR_ARGUMENTS_BAD)
+			TC_TOKEN_ERR (CKR_NO_EVENT)
+			TC_TOKEN_ERR (CKR_NEED_TO_CREATE_THREADS)
+			TC_TOKEN_ERR (CKR_CANT_LOCK)
+			TC_TOKEN_ERR (CKR_ATTRIBUTE_READ_ONLY)
+			TC_TOKEN_ERR (CKR_ATTRIBUTE_SENSITIVE)
+			TC_TOKEN_ERR (CKR_ATTRIBUTE_TYPE_INVALID)
+			TC_TOKEN_ERR (CKR_ATTRIBUTE_VALUE_INVALID)
+			TC_TOKEN_ERR (CKR_DATA_INVALID)
+			TC_TOKEN_ERR (CKR_DATA_LEN_RANGE)
+			TC_TOKEN_ERR (CKR_DEVICE_ERROR)
+			TC_TOKEN_ERR (CKR_DEVICE_MEMORY)
+			TC_TOKEN_ERR (CKR_DEVICE_REMOVED)
+			TC_TOKEN_ERR (CKR_ENCRYPTED_DATA_INVALID)
+			TC_TOKEN_ERR (CKR_ENCRYPTED_DATA_LEN_RANGE)
+			TC_TOKEN_ERR (CKR_FUNCTION_CANCELED)
+			TC_TOKEN_ERR (CKR_FUNCTION_NOT_PARALLEL)
+			TC_TOKEN_ERR (CKR_FUNCTION_NOT_SUPPORTED)
+			TC_TOKEN_ERR (CKR_KEY_HANDLE_INVALID)
+			TC_TOKEN_ERR (CKR_KEY_SIZE_RANGE)
+			TC_TOKEN_ERR (CKR_KEY_TYPE_INCONSISTENT)
+			TC_TOKEN_ERR (CKR_KEY_NOT_NEEDED)
+			TC_TOKEN_ERR (CKR_KEY_CHANGED)
+			TC_TOKEN_ERR (CKR_KEY_NEEDED)
+			TC_TOKEN_ERR (CKR_KEY_INDIGESTIBLE)
+			TC_TOKEN_ERR (CKR_KEY_FUNCTION_NOT_PERMITTED)
+			TC_TOKEN_ERR (CKR_KEY_NOT_WRAPPABLE)
+			TC_TOKEN_ERR (CKR_KEY_UNEXTRACTABLE)
+			TC_TOKEN_ERR (CKR_MECHANISM_INVALID)
+			TC_TOKEN_ERR (CKR_MECHANISM_PARAM_INVALID)
+			TC_TOKEN_ERR (CKR_OBJECT_HANDLE_INVALID)
+			TC_TOKEN_ERR (CKR_OPERATION_ACTIVE)
+			TC_TOKEN_ERR (CKR_OPERATION_NOT_INITIALIZED)
+			TC_TOKEN_ERR (CKR_PIN_INCORRECT)
+			TC_TOKEN_ERR (CKR_PIN_INVALID)
+			TC_TOKEN_ERR (CKR_PIN_LEN_RANGE)
+			TC_TOKEN_ERR (CKR_PIN_EXPIRED)
+			TC_TOKEN_ERR (CKR_PIN_LOCKED)
+			TC_TOKEN_ERR (CKR_SESSION_CLOSED)
+			TC_TOKEN_ERR (CKR_SESSION_COUNT)
+			TC_TOKEN_ERR (CKR_SESSION_HANDLE_INVALID)
+			TC_TOKEN_ERR (CKR_SESSION_PARALLEL_NOT_SUPPORTED)
+			TC_TOKEN_ERR (CKR_SESSION_READ_ONLY)
+			TC_TOKEN_ERR (CKR_SESSION_EXISTS)
+			TC_TOKEN_ERR (CKR_SESSION_READ_ONLY_EXISTS)
+			TC_TOKEN_ERR (CKR_SESSION_READ_WRITE_SO_EXISTS)
+			TC_TOKEN_ERR (CKR_SIGNATURE_INVALID)
+			TC_TOKEN_ERR (CKR_SIGNATURE_LEN_RANGE)
+			TC_TOKEN_ERR (CKR_TEMPLATE_INCOMPLETE)
+			TC_TOKEN_ERR (CKR_TEMPLATE_INCONSISTENT)
+			TC_TOKEN_ERR (CKR_TOKEN_NOT_PRESENT)
+			TC_TOKEN_ERR (CKR_TOKEN_NOT_RECOGNIZED)
+			TC_TOKEN_ERR (CKR_TOKEN_WRITE_PROTECTED)
+			TC_TOKEN_ERR (CKR_UNWRAPPING_KEY_HANDLE_INVALID)
+			TC_TOKEN_ERR (CKR_UNWRAPPING_KEY_SIZE_RANGE)
+			TC_TOKEN_ERR (CKR_UNWRAPPING_KEY_TYPE_INCONSISTENT)
+			TC_TOKEN_ERR (CKR_USER_ALREADY_LOGGED_IN)
+			TC_TOKEN_ERR (CKR_USER_NOT_LOGGED_IN)
+			TC_TOKEN_ERR (CKR_USER_PIN_NOT_INITIALIZED)
+			TC_TOKEN_ERR (CKR_USER_TYPE_INVALID)
+			TC_TOKEN_ERR (CKR_USER_ANOTHER_ALREADY_LOGGED_IN)
+			TC_TOKEN_ERR (CKR_USER_TOO_MANY_TYPES)
+			TC_TOKEN_ERR (CKR_WRAPPED_KEY_INVALID)
+			TC_TOKEN_ERR (CKR_WRAPPED_KEY_LEN_RANGE)
+			TC_TOKEN_ERR (CKR_WRAPPING_KEY_HANDLE_INVALID)
+			TC_TOKEN_ERR (CKR_WRAPPING_KEY_SIZE_RANGE)
+			TC_TOKEN_ERR (CKR_WRAPPING_KEY_TYPE_INCONSISTENT)
+			TC_TOKEN_ERR (CKR_RANDOM_SEED_NOT_SUPPORTED)
+			TC_TOKEN_ERR (CKR_RANDOM_NO_RNG)
+			TC_TOKEN_ERR (CKR_DOMAIN_PARAMS_INVALID)
+			TC_TOKEN_ERR (CKR_BUFFER_TOO_SMALL)
+			TC_TOKEN_ERR (CKR_SAVED_STATE_INVALID)
+			TC_TOKEN_ERR (CKR_INFORMATION_SENSITIVE)
+			TC_TOKEN_ERR (CKR_STATE_UNSAVEABLE)
+			TC_TOKEN_ERR (CKR_CRYPTOKI_NOT_INITIALIZED)
+			TC_TOKEN_ERR (CKR_CRYPTOKI_ALREADY_INITIALIZED)
+			TC_TOKEN_ERR (CKR_MUTEX_BAD)
+			TC_TOKEN_ERR (CKR_MUTEX_NOT_LOCKED)
+			TC_TOKEN_ERR (CKR_NEW_PIN_MODE)
+			TC_TOKEN_ERR (CKR_NEXT_OTP)
+			TC_TOKEN_ERR (CKR_FUNCTION_REJECTED)
 
-		default:
-			{
-				stringstream s;
-				s << "0x" << hex << ErrorCode;
-				return s.str();
-			}
+#undef		TC_TOKEN_ERR
 		};
 
-#undef TC_CASE_STR
+
+		for (size_t i = 0; i < array_capacity (ErrorStrings); ++i)
+		{
+			if (ErrorStrings[i].ErrorCode == ErrorCode)
+				return ErrorStrings[i].ErrorString;
+		}
+
+		stringstream s;
+		s << "0x" << hex << ErrorCode;
+		return s.str();
+
 	}
 
 #ifdef TC_HEADER_Common_Exception
