@@ -4,8 +4,8 @@
  Copyright (c) 1998-2000 Paul Le Roux and which is governed by the 'License
  Agreement for Encryption for the Masses'. Modifications and additions to
  the original source code (contained in this file) and all other portions of
- this file are Copyright (c) 2003-2008 TrueCrypt Foundation and are governed
- by the TrueCrypt License 2.6 the full text of which is contained in the
+ this file are Copyright (c) 2003-2009 TrueCrypt Foundation and are governed
+ by the TrueCrypt License 2.7 the full text of which is contained in the
  file License.txt included in TrueCrypt binary and source code distribution
  packages. */
 
@@ -1059,13 +1059,14 @@ BOOL TestLegacySectorBufEncryption (PCRYPTO_INFO ci)
 		if (EAInit (ci->ea, (unsigned char *)buf, ci->ks) == ERR_CIPHER_INIT_FAILURE)
 			return FALSE;
 
-		// Test all available modes of operation
+		// Test all deprecated modes of operation
 		for (ci->mode = EAGetFirstMode (ci->ea);
 			ci->mode != 0;
 			ci->mode = EAGetNextMode (ci->ea, ci->mode))
 		{
+			// Skip modes that are not deprecated
 			if (ci->mode == XTS)
-				continue;	// XTS mode is tested in another function
+				continue;
 
 			if (ci->mode == LRW
 				&& (blockSize == 8 && !lrw64InitDone || blockSize == 16 && !lrw128InitDone ))
