@@ -4,8 +4,8 @@
  Copyright (c) 1998-2000 Paul Le Roux and which is governed by the 'License
  Agreement for Encryption for the Masses'. Modifications and additions to
  the original source code (contained in this file) and all other portions of
- this file are Copyright (c) 2003-2008 TrueCrypt Foundation and are governed
- by the TrueCrypt License 2.6 the full text of which is contained in the
+ this file are Copyright (c) 2003-2009 TrueCrypt Foundation and are governed
+ by the TrueCrypt License 2.7 the full text of which is contained in the
  file License.txt included in TrueCrypt binary and source code distribution
  packages. */
 
@@ -68,6 +68,8 @@ NTSTATUS TCOpenVolume (PDEVICE_OBJECT DeviceObject,
 			goto error;
 	}
 
+	mount->VolumeMountedReadOnlyAfterDeviceWriteProtected = FALSE;
+
 	// If we are opening a device, query its size first
 	if (bRawDevice)
 	{
@@ -117,6 +119,7 @@ NTSTATUS TCOpenVolume (PDEVICE_OBJECT DeviceObject,
 		{
 			mount->bMountReadOnly = TRUE;
 			DeviceObject->Characteristics |= FILE_READ_ONLY_DEVICE;
+			mount->VolumeMountedReadOnlyAfterDeviceWriteProtected = TRUE;
 		}
 
 		ProbingHostDeviceForWrite = FALSE;
@@ -284,6 +287,7 @@ NTSTATUS TCOpenVolume (PDEVICE_OBJECT DeviceObject,
 	}
 
 	Extension->DiskLength = lDiskLength.QuadPart;
+	Extension->HostLength = lDiskLength.QuadPart;
 
 	readBuffer = TCalloc (max (TC_VOLUME_HEADER_EFFECTIVE_SIZE, PAGE_SIZE));
 	if (readBuffer == NULL)
