@@ -1,7 +1,7 @@
 /*
  Copyright (c) 2005 TrueCrypt Foundation. All rights reserved.
 
- Governed by the TrueCrypt License 2.6 the full text of which is contained
+ Governed by the TrueCrypt License 2.7 the full text of which is contained
  in the file License.txt included in TrueCrypt binary and source code
  distribution packages.
 */
@@ -19,7 +19,9 @@ enum
 	HandleHotKey()	*/
 
 	HK_AUTOMOUNT_DEVICES = 0,
+	HK_CLOSE_SECURITY_TOKEN_SESSIONS,
 	HK_DISMOUNT_ALL,
+	HK_DISMOUNT_ALL_AND_WIPE,
 	HK_FORCE_DISMOUNT_ALL_AND_WIPE,
 	HK_FORCE_DISMOUNT_ALL_AND_WIPE_AND_EXIT,
 	HK_MOUNT_FAVORITE_VOLUMES,
