@@ -1,7 +1,7 @@
 /*
- Copyright (c) 2008 TrueCrypt Foundation. All rights reserved.
+ Copyright (c) 2008-2009 TrueCrypt Foundation. All rights reserved.
 
- Governed by the TrueCrypt License 2.6 the full text of which is contained
+ Governed by the TrueCrypt License 2.7 the full text of which is contained
  in the file License.txt included in TrueCrypt binary and source code
  distribution packages.
 */
@@ -19,7 +19,7 @@ namespace TrueCrypt
 	public:
 		static void Create (const DirectoryPath &path);
 		static DirectoryPath AppendSeparator (const DirectoryPath &path);
-		static FilePathList GetFilePaths (const DirectoryPath &path = L".");
+		static FilePathList GetFilePaths (const DirectoryPath &path = L".", bool regularFilesOnly = true);
 
 	private:
 		Directory ();
