@@ -1,7 +1,7 @@
 /*
- Copyright (c) 2008 TrueCrypt Foundation. All rights reserved.
+ Copyright (c) 2008-2009 TrueCrypt Foundation. All rights reserved.
 
- Governed by the TrueCrypt License 2.6 the full text of which is contained
+ Governed by the TrueCrypt License 2.7 the full text of which is contained
  in the file License.txt included in TrueCrypt binary and source code
  distribution packages.
 */
@@ -17,7 +17,7 @@ namespace TrueCrypt
 	class SystemException : public Exception
 	{
 	public:
-		SystemException () { }
+		SystemException ();
 		SystemException (const string &message);
 		SystemException (const string &message, const string &subject);
 		SystemException (const string &message, const wstring &subject);
